/*******************************************************************************

    This file is a part of Fahrplan for maemo 2009-2010

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QtGui>
#include <QtMaemo5>
#include <qgtkstyle.h>
#include <QMaemo5InformationBox>
#include <QtCore/QtGlobal>
#include <QtCore/QtDebug>

#include <QtCore/QCoreApplication>
#include <QtCore/QBuffer>
#include <QtCore/QStringList>
#include <qgeopositioninfo.h>
#include <qgeopositioninfosource.h>

#include "parser_mobilebahnde.h"
#include "parser_mvvde.h"
#include "selectstationdialog.h"
#include "configdialog.h"

#define APP_VER "0.0.11"

//Use the QtMobility namespace
QTM_USE_NAMESPACE

class ResultItemUserData : public QObjectUserData
{
public:
    ResultItem item;
};

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent=0);
    void initBackend();

public slots:
    void positionUpdated(QGeoPositionInfo geoPositionInfo);

private slots:
    void searchButtonClicked(bool checked = false);
    void aboutButtonClicked();
    void settingsButtonClicked();
    void earlierButtonClicked();
    void laterButtonClicked();
    void stationDepartureClicked();
    void stationArrivalClicked();
    void swapClicked();
    void gpsButtonClicked();
    void windowDestroyed(QObject* obj = 0);
    void resultItemClicked();

private:
    QPushButton        *searchButton;
    QMaemo5ValueButton *stationDeparture;
    QMaemo5ValueButton *stationArrival;
    QMaemo5ValueButton *dateSelector;
    QMaemo5ValueButton *timeSelector;
    QMaemo5ValueButton *modeButton;
    ResultInfo         lastSearchResult;
    DetailResultInfo   lastSearchDetails;
    QPointer<QMainWindow> resultWindow;
    QPointer<QMainWindow> detailsWindow;
    QGeoPositionInfo myPositionInfo;
    QPointer<QGeoPositionInfoSource> locationDataSource;
    QSettings          *settings;
    bool locationDataSourceIsUpdating;

    parserAbstract *backendParser;

    void showResultWindow();
    void showDetailsWindow();
    bool eventFilter(QObject* pObject, QEvent* pEvent);
};

#endif // MAINWINDOW_H
